/*
 * Purple - Internet Messaging Library
 * Copyright (C) Pidgin Developers <devel@pidgin.im>
 *
 * Purple is the legal property of its developers, whose names are too numerous
 * to list here. Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this library; if not, see <https://www.gnu.org/licenses/>.
 */

#if !defined(PURPLE_GLOBAL_HEADER_INSIDE) && !defined(PURPLE_COMPILATION)
# error "only <purple.h> may be included directly"
#endif

#ifndef PURPLE_HISTORY_MANAGER_H
#define PURPLE_HISTORY_MANAGER_H

#include <glib.h>
#include <glib-object.h>

#include "purplehistoryadapter.h"
#include "purpleversion.h"

G_BEGIN_DECLS

/**
 * PURPLE_HISTORY_MANAGER_DOMAIN:
 *
 * A #GError domain for errors from #PurpleHistoryManager.
 *
 * Since: 3.0
 */
#define PURPLE_HISTORY_MANAGER_DOMAIN \
	g_quark_from_static_string("purple-history-manager") \
	PURPLE_AVAILABLE_MACRO_IN_3_0

#define PURPLE_TYPE_HISTORY_MANAGER (purple_history_manager_get_type())

PURPLE_AVAILABLE_IN_3_0
G_DECLARE_FINAL_TYPE(PurpleHistoryManager, purple_history_manager, PURPLE,
                     HISTORY_MANAGER, GObject)

/**
 * PurpleHistoryManager:
 *
 * #PurpleHistoryManager keeps track of all adapters and emits signals when
 * adapters are added and removed.
 *
 * Since: 3.0
 */

/**
 * purple_history_manager_startup:
 * @adapter: (transfer full) (nullable): The history adapter to use by default.
 * @error: A return address for a #GError.
 *
 * Starts up the history manager by creating the default instance and setting
 * @adapter as active if @adapter is non %NULL.
 *
 * Returns: %TRUE if startup was successful, otherwise %FALSE with @error
 *          potentially set.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_history_manager_startup(PurpleHistoryAdapter *adapter, GError **error);

/**
 * purple_history_manager_shutdown:
 *
 * Shuts down the history manager by destroying the default instance.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
void purple_history_manager_shutdown(void);

/**
 * purple_history_manager_get_default:
 *
 * Gets the default #PurpleHistoryManager instance.
 *
 * Returns: (transfer none): The default #PurpleHistoryManager instance.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
PurpleHistoryManager *purple_history_manager_get_default(void);

/**
 * purple_history_manager_get_default_as_model:
 *
 * Gets the default #PurpleHistoryManager instance cast to a
 * [iface@Gio.ListModel].
 *
 * Returns: (transfer none): The default #PurpleHistoryManager instance cast to
 *          a model.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
GListModel *purple_history_manager_get_default_as_model(void);

/**
 * purple_history_manager_get_active:
 * @manager: The #PurpleHistoryManager instance.
 *
 * Gets the active #PurpleHistoryAdapter instance.
 *
 * Returns: (transfer none): The active @adapter
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
PurpleHistoryAdapter *purple_history_manager_get_active(PurpleHistoryManager *manager);

 /**
 * purple_history_manager_set_active:
 * @manager: The #PurpleHistoryManager instance.
 * @id: The id of the #PurpleHistoryAdapter to set active.
 * @error: A return address for a #GError.
 *
 * Sets the active #PurpleHistoryAdapter instance.
 *
 * Returns: %TRUE if setting the @adapter was successful with @manager
 *          %FALSE otherwise.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_history_manager_set_active(PurpleHistoryManager *manager, const gchar *id, GError **error);

/**
 * purple_history_manager_add:
 * @manager: The #PurpleHistoryManager instance.
 * @adapter: The #PurpleHistoryAdapter to add.
 * @error: A return address for a #GError.
 *
 * Adds @adapter to @manager.
 *
 * Returns: %TRUE if @adapter was successfully added to @manager, %FALSE
 *          otherwise.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_history_manager_add(PurpleHistoryManager *manager, PurpleHistoryAdapter *adapter, GError **error);

/**
 * purple_history_manager_remove:
 * @manager: The #PurpleHistoryManager instance.
 * @adapter: The #PurpleHistoryAdapter to remove.
 * @error: A return address for a #GError.
 *
 * Removes @adapter from @manager.
 *
 * Returns: %TRUE if @adapter was successfully removed from @manager, %FALSE
 *          otherwise.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_history_manager_remove(PurpleHistoryManager *manager, PurpleHistoryAdapter *adapter, GError **error);

/**
 * purple_history_manager_query:
 * @manager: The #PurpleHistoryManager instance.
 * @query: A query to send to the @manager instance.
 * @error: A return address for a #GError.
 *
 * Sends a query to the #PurpleHistoryAdapter @manager instance.
 *
 * Returns: (transfer full) (element-type PurpleHistoryAdapter): The list
 *          containing all of the #PurpleMessage's that matched the query
 *          with @manager.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
GList *purple_history_manager_query(PurpleHistoryManager *manager, const gchar *query, GError **error);

/**
 * purple_history_manager_remove_query:
 * @manager: The #PurpleHistoryManager instance.
 * @query: A query to send to the @manager instance.
 * @error: A return address for a #GError.
 *
 * Removes messages from the active #PurpleHistoryAdapter of @manager that
 * match @query.
 *
 * Returns: %TRUE if messages matching @query were successfully removed from
 *          the active adapter of @manager, %FALSE otherwise.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_history_manager_remove_query(PurpleHistoryManager *manager, const gchar *query, GError **error);

/**
 * purple_history_manager_write:
 * @manager: The #PurpleHistoryManager instance.
 * @conversation: The #PurpleConversation.
 * @message: The #PurpleMessage to pass to the @manager.
 * @error: A return address for a #GError.
 *
 * Writes @message to the active adapter of @manager.
 *
 * Returns: %TRUE if @message was successfully written, %FALSE otherwise.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_history_manager_write(PurpleHistoryManager *manager, PurpleConversation *conversation, PurpleMessage *message, GError **error);

G_END_DECLS

#endif /* PURPLE_HISTORY_MANAGER_H */
